/* --------------------------------------------------------- */
/*	BASIC-E													 */
/*	Copyright(C)2006 t.hara									 */
/* ========================================================= */
/*	function												 */
/* --------------------------------------------------------- */

#include "system_resource.h"
#include "parser.h"
#include "interpreter.h"
#include "function.h"
#include "zb_error.h"
#include <cstring>
#include <cstdlib>
#include <cmath>

namespace {

	/* --------------------------------------------------------- */
	void fnc_rnd( c_value& term_value, const unsigned char* &, c_system_resource * ) {

		term_value.change_real( (double)rand() / RAND_MAX );
	}

	/* --------------------------------------------------------- */
	void fnc_sin( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( sin( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_cos( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( cos( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_tan( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( tan( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_log( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( log( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_sqrt( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( sqrt( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_exp( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( exp( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_log10( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( log10( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_abs( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( abs( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_asin( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( asin( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_acos( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( acos( value.get()->get_real() ) );
	}
	/* --------------------------------------------------------- */
	void fnc_atan( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( atan( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_sinh( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( sinh( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_cosh( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( cosh( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_tanh( c_value& term_value, const unsigned char* &p_code, c_system_resource *p ) {
		c_expression expression( p );
		c_value value;

		p->error_code = expression.expression( value, p_code );
		term_value.change_real( tanh( value.get()->get_real() ) );
	}

	/* --------------------------------------------------------- */
	void fnc_inkey( c_value& term_value, const unsigned char* &, c_system_resource *p ) {
		
		term_value.change_integer( p->p_keyboard->pop_key() );
	}
}

/* --------------------------------------------------------- */
const c_function_list c_function::function_list[] = {
	{	"rnd"		,	fnc_rnd			,	c_parser::PC_RNUMBER	},
	{	"sin"		,	fnc_sin			,	c_parser::PC_RNUMBER	},
	{	"cos"		,	fnc_cos			,	c_parser::PC_RNUMBER	},
	{	"tan"		,	fnc_tan			,	c_parser::PC_RNUMBER	},
	{	"log"		,	fnc_log			,	c_parser::PC_RNUMBER	},
	{	"sqrt"		,	fnc_sqrt		,	c_parser::PC_RNUMBER	},
	{	"exp"		,	fnc_exp			,	c_parser::PC_RNUMBER	},
	{	"log10"		,	fnc_log10		,	c_parser::PC_RNUMBER	},
	{	"abs"		,	fnc_abs			,	c_parser::PC_RNUMBER	},
	{	"asin"		,	fnc_asin		,	c_parser::PC_RNUMBER	},
	{	"acos"		,	fnc_acos		,	c_parser::PC_RNUMBER	},
	{	"atan"		,	fnc_atan		,	c_parser::PC_RNUMBER	},
	{	"sinh"		,	fnc_sinh		,	c_parser::PC_RNUMBER	},
	{	"cosh"		,	fnc_cosh		,	c_parser::PC_RNUMBER	},
	{	"tanh"		,	fnc_tanh		,	c_parser::PC_RNUMBER	},
	{	"inkey"		,	fnc_inkey		,	c_parser::PC_INUMBER	},
};

/* --------------------------------------------------------- */
int c_function::get_code( const char* p_name ) {
	unsigned int i;

	for( i = 0; i < sizeof(function_list)/sizeof(c_function_list); i++ ) {
		if( strcmp( p_name, function_list[i].p_name ) == 0 ) {
			return i;
		}
	}
	return -1;	/* syntax error */
}
