/* --------------------------------------------------------- */
/*	BASIC-E													 */
/*	Copyright(C)2006 t.hara									 */
/* ========================================================= */
/*	interpreter												 */
/* --------------------------------------------------------- */

#ifndef __INTERPRETER_H__
#define __INTERPRETER_H__

#include "parser.h"
#include "program_list.h"
#include "variant_value.h"
#include "command.h"
#include "function.h"
#include "stack.h"
#include "variable_manager.h"

/* --------------------------------------------------------- */
class c_expression {
private:
	class c_system_resource	*p_res;

	c_function	function;

public:
	c_expression( c_system_resource *p ) {
		p_res = p;
	}
	
	int class_change( c_value& result, int operand_type );

	int term( c_value& term_value, const unsigned char* &p_code, bool &f_continue );
	int addision( c_value& result, const unsigned char* &p_code, bool &f_continue );
	int multiplication( c_value& result, const unsigned char* &p_code, bool &f_continue );
	int comparator( c_value& result, const unsigned char* &p_code, bool &f_continue );
	int and( c_value& result, const unsigned char* &p_code, bool &f_continue );
	int or( c_value& result, const unsigned char* &p_code, bool &f_continue );

	int expression( c_value& result, const unsigned char* &p_code );
};

/* --------------------------------------------------------- */
struct c_for_info {
	c_variable_integer	*p_instance;
	int					next_step;
	int					end_point;
};

/* --------------------------------------------------------- */
class c_interpreter {
private:
	class c_system_resource	*p_res;

	c_command				command;
	c_function				function;
	c_stack<c_for_info>		for_info;

	void cmd_goto( const unsigned char* &p_code );
	void cmd_if( const unsigned char* &p_code );
	void cmd_for( const unsigned char* &p_code );
	void cmd_next( const unsigned char* &p_code );

	void process_ivar( const unsigned char* &p_code );
	void process_rvar( const unsigned char* &p_code );
	void process_svar( const unsigned char* &p_code );
	void process_iarray( const unsigned char* &p_code );
	void process_rarray( const unsigned char* &p_code );
	void process_sarray( const unsigned char* &p_code );

public:
	c_program_list		list;

	c_interpreter( void ) {
	}

	void set_system_resource( c_system_resource *p ) {
		p_res = p;
		list.set_system_resource( p );
	}
	int run( c_parser *p_line );

	int get_function_code( const char* p_name ) {
		return function.get_code( p_name );
	}

	int get_command_code( const char* p_name ) {
		return command.get_code( p_name );
	}

	int dimension_check( const unsigned char* &p_code, int *p_elements );
	void dimension_alloc( const unsigned char* &p_code );
	void dimension_free( void );
};

#endif
