/* --------------------------------------------------------- */
/*	BASIC-E TEXT layer										 */
/*	Copyright(C)2006 t.hara									 */
/* ========================================================= */
/*	text screen layer										 */
/* --------------------------------------------------------- */

#ifndef __TEXT_LAYER_H__
#define __TEXT_LAYER_H__

#include <cstring>

class c_text_layer {
private:
	int columns;
	int rows;
	int width;
	int height;
	int line_step;

	/* (MSB) {rrrrrggggggbbbbbXXXXXXXRXccccccc} (LSB)	*/
	unsigned int *p_text_vram;
	
public:
	c_text_layer( void ): columns(0), rows(0), p_text_vram(0) {
	}
	
	~c_text_layer() {
		release();
	}

	void create( int width, int height, int step ) {
		release();
		this->width		= width;
		this->height	= height;
		line_step		= step / 2;
		columns			= height / 8;
		rows			= width  / 16;
		p_text_vram = new unsigned int [ columns * rows + 1 ];	/* "+1" is guardian for delete_char(). */
		clear();
	}

	void clear( void ) {
		std::memset( p_text_vram, 0, ( columns * rows + 1 ) * sizeof( unsigned int ) );
	}

	void release( void ) {
		delete[] p_text_vram;
		p_text_vram = 0;
	}

	int get_rows( void ) const {
		return rows;
	}

	int get_columns( void ) const {
		return columns;
	}

	unsigned int get_char( int column, int row ) const {
		return p_text_vram[ column + row * columns ];
	}
	
	unsigned int get_char( int index ) const {
		return p_text_vram[ index ];
	}
	
	void put_char( int column, int row, unsigned int c ) {
		int address;

		address = column + row * columns;
		if( address ) {
			for( int i = 1; i <= column; i++ ) {
				if( (p_text_vram[ address - i ] & 0xFF) != 0 ) {
					break;
				}
				p_text_vram[ address - i ] |= 0x20;
			}
		}
		p_text_vram[ column + row * columns ] = c;
	}

	void draw( unsigned short *p_vram );

	void scroll_up( void );
	void delete_char( int column, int row );
};

#endif
